//
//  ViewController.swift
//  FlowGesture
//
//  Created by Travis on 2019-01-04.
//  Copyright © 2019 Travis. All rights reserved.
//

import UIKit

class ViewController: UIViewController {
    //since we created 3 timelines from the same project, we can use a single scene
    let scene = SceneView(frame: CGRect(origin: CGPoint.zero, size: SceneView.Defaults.size))

    //a reference to whichever timeline is currently being run
    var currentTimeline: Timeline?

    //a flag to determine if the current timeline should be reset
    var shouldReset = false

    override func viewDidLoad() {
        super.viewDidLoad()

        //add the scene to the app's main view
        view.addSubview(scene)

        //center the scene to the app's main view
        scene.center = view.center

        //add a tap gesture to the scene, passing in the shrink method
        let tap = UITapGestureRecognizer(target: self, action: #selector(shrink))
        scene.addGestureRecognizer(tap)

        //add a swipe gesture to the scene, passing in the swipe method
        let swipe = UISwipeGestureRecognizer(target: self, action: #selector(move))
        scene.addGestureRecognizer(swipe)

        let doubleTap = UITapGestureRecognizer(target: self, action: #selector(rotate))
        scene.addGestureRecognizer(doubleTap)
        doubleTap.numberOfTapsRequired = 2
        tap.require(toFail: doubleTap)
    }

    //resets the current timeline
    func reset() {
        currentTimeline?.reset()
        shouldReset = false
    }

    //runs the shrink timeline
    func animateShrink() {
        currentTimeline = ShrinkTimeline(view: scene)
        currentTimeline?.animate()
        shouldReset = true
    }

    //shrink or reset
    @objc func shrink() {
        if shouldReset { reset() }
        else { animateShrink() }
    }

    //move or reset
    @objc func move() {
        if shouldReset { reset() }
        else { animateMove() }
    }

    //runs the move timeline
    func animateMove() {
        currentTimeline = ShrinkMoveTimeline(view: scene)
        currentTimeline?.animate()
        shouldReset = true
    }

    //runs the rotate timeline
    func animateRotate() {
        currentTimeline = ShrinkRotateTimeline(view: scene)
        currentTimeline?.animate()
        shouldReset = true
    }

    //rotate or reset
    @objc func rotate() {
        if shouldReset { reset() }
        else { animateRotate() }
    }

    override var prefersStatusBarHidden: Bool {
        return true
    }
}
